﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using NST.Auth.DataContracts;

namespace NST.Auth.AspNetDb
{
    internal class UserInfoAssembler<T> where T : UserInfoDC, new()
    {
        internal static IList<T> AssembleContracts(IQueryable<aspnet_User> froms)
        {
            IList<T> tos = new List<T>();

            foreach (aspnet_User from in froms)
            {
                tos.Add(AssembleContract(from));
            }

            return tos;
        }

        internal static T AssembleContract(aspnet_User objFrom)
        {
            T to = new T();

            to.Id = objFrom.UserId;
            to.ApplicationId = objFrom.ApplicationId;
            to.Name = objFrom.UserName;            
            to.MobileAlias = objFrom.MobileAlias;
            to.IsAnonymous = objFrom.IsAnonymous;
            to.LastActivityDate = objFrom.LastActivityDate;

            if (objFrom.aspnet_Membership != null)
            {
                to.Email = objFrom.aspnet_Membership.Email;
                to.Comment = objFrom.aspnet_Membership.Comment;
                to.Password = objFrom.aspnet_Membership.Password;
                to.PasswordAnswer = objFrom.aspnet_Membership.PasswordAnswer;
                to.PasswordQuestion = objFrom.aspnet_Membership.PasswordQuestion;
                to.IsApproved = objFrom.aspnet_Membership.IsApproved;
                to.IsLockedOut = objFrom.aspnet_Membership.IsLockedOut;
                to.CreateDate = objFrom.aspnet_Membership.CreateDate;
                to.LastLockoutDate = objFrom.aspnet_Membership.LastLockoutDate;
                to.LastLoginDate = objFrom.aspnet_Membership.LastLoginDate;
                to.LastPasswordChangedDate = objFrom.aspnet_Membership.LastPasswordChangedDate;
            }

            if (objFrom.aspnet_UsersInRoles != null && objFrom.aspnet_UsersInRoles.Count > 0)
            {
                to.Roles = RoleAssembler<RoleDC>.AssembleContracts(objFrom.aspnet_UsersInRoles);
            }

            return to;
        }

    }
}
