﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web.Security;
using System.Diagnostics;
using System.Reflection;
using System.Web.Configuration;
using System.Configuration.Provider;
using System.Configuration;
using System.Collections.Specialized;
using System.Data.Linq;
using NST.Auth.ProviderContracts;

namespace NST.Auth.AspNetDb
{
    public class AuthMembershipProvider : SqlMembershipProvider
    {
         static MachineKeySection machineKey;

        /*************************************************************************
         * General settings
         *************************************************************************/

        private string _applicationName;
        public override string ApplicationName
        {
            get { return _applicationName; }
            set { _applicationName = value; }
        }


        /*************************************************************************
         * Private settings
         *************************************************************************/

        private string _providerName;
        public string ProviderName
        {
            get { return _providerName; }
        }

        private TimeSpan _userIsOnlineTimeWindow;
        public TimeSpan UserIsOnlineTimeWindow
        {
            get { return _userIsOnlineTimeWindow; }
        }


        static AuthMembershipProvider()
        {
            
        }

        public AuthMembershipProvider()
        {

        }

        public AuthMembershipProvider(string applicationName)
        {
            ApplicationName = applicationName;
        }

    //    #region MembershipProvider Method Overrides

        public bool ChangePassword(string applicationName, string userName, string oldPassword, string newPassword)
        {
            MembershipUser _membershipUser = base.GetUser(userName, false);
            return _membershipUser.ChangePassword(_membershipUser.GetPassword(), newPassword);            
        }

        public bool ChangePasswordQuestionAndAnswer(string applicationName, string userName, string password, string passwordQuestion, string passwordAnswer)
        {
            MembershipUser membershipUser = base.GetUser(userName, false);
            return membershipUser.ChangePasswordQuestionAndAnswer(password, passwordQuestion, passwordAnswer);
        }

        //public override MembershipUser CreateUser(string username, string password, string email, string passwordQuestion, string passwordAnswer, 
        //    bool isApproved, object providerUserKey, out MembershipCreateStatus status)
        //{
        //    try
        //    {
        //         return base.CreateUser(username, passwordAnswer, email, passwordQuestion, passwordAnswer, isApproved, out status);
        //    }
        //    catch (Exception ex)
        //    {
        //        throw new Exception(string.Format("{0}(): Exception occurred {1}. ", this.GetType().ToString(), ex.Message));
        //    }
        //}

        public override bool DeleteUser(string username, bool deleteAllRelatedData)
        {
            try
            {
                return base.DeleteUser(username, deleteAllRelatedData);
            }
            catch (Exception ex)
            {
                throw new Exception(string.Format("{0}(): Exception occurred {1}. ", this.GetType().ToString(), ex.Message));
            }
        }

        public MembershipUserCollection FindUsersByEmail(string applicationName, string emailToMatch, int pageIndex, int pageSize, out int totalRecords)
        {
            try
            {
                return base.FindUsersByEmail(emailToMatch, pageIndex, pageSize, out totalRecords);
            }
            catch (Exception ex)
            {
                throw new Exception(string.Format("{0}(): Exception occurred {1}. ", this.GetType().ToString(), ex.Message));
            }
        }

        public MembershipUserCollection FindUsersByName(string applicationName, string usernameToMatch, int pageIndex, int pageSize, out int totalRecords)
        {
            try
            {
                return base.FindUsersByName(usernameToMatch, pageIndex, pageSize, out totalRecords);
            }
            catch (Exception ex)
            {
                throw new Exception(string.Format("{0}(): Exception occurred {1}. ", this.GetType().ToString(), ex.Message));
            }
        }

        public MembershipUserCollection GetAllUsers(string applicationName, int pageIndex, int pageSize, out int totalRecords)
        {
            try
            {
                return base.GetAllUsers(pageIndex, pageSize, out totalRecords);
            }
            catch (Exception ex)
            {
                throw new Exception(string.Format("{0}(): Exception occurred {1}. ", this.GetType().ToString(), ex.Message));
            }
        }

        public int GetNumberOfUsersOnline(string applicationName)
        {
            try
            {
                return base.GetNumberOfUsersOnline();
            }
            catch (Exception ex)
            {
                throw new Exception(string.Format("{0}(): Exception occurred {1}. ", this.GetType().ToString(), ex.Message));
            }
        }

        public string GetPassword(string applicationName, string userName, string answer)
        {
            try
            {
                MembershipUser _mu = base.GetUser(userName, false);
                return _mu.GetPassword(answer);
            }
            catch (Exception ex)
            {
                throw new Exception(string.Format("{0}(): Exception occurred {1}. ", this.GetType().ToString(), ex.Message));
            }
        }

        public MembershipUser GetUser(string applicationName, string username, bool userIsOnline)
        {
            try
            {
                return base.GetUser(username, userIsOnline);
            }
            catch (Exception ex)
            {
                throw new Exception(string.Format("{0}(): Exception occurred {1}. ", this.GetType().ToString(), ex.Message));
            }
        }

        public MembershipUser GetUser(string applicationName, object providerUserKey, bool userIsOnline)
        {
            try
            {
                return base.GetUser(providerUserKey, userIsOnline);
            }
            catch (Exception ex)
            {
                throw new Exception(string.Format("{0}(): Exception occurred {1}. ", this.GetType().ToString(), ex.Message));
            }
        }

        public string GetUserNameByEmail(string applicationName, string email)
        {
            try
            {
                return base.GetUserNameByEmail(email);
            }
            catch (Exception ex)
            {
                throw new Exception(string.Format("{0}(): Exception occurred {1}. ", this.GetType().ToString(), ex.Message));
            }
        }

    //    /*************************************************************************
    //     * Class initialization
    //     *************************************************************************/

        public override void Initialize(string name, NameValueCollection config)
        {
            if (config == null)
                throw new ArgumentNullException("config");

            if (name == null || name.Length == 0)
                name = "AuthMembershipProvider";

            if (String.IsNullOrEmpty(config["description"]))
            {
                config.Remove("description");
                config.Add("description", "Authenticator Membership Provider");
            }

            // Initialize base class
            base.Initialize(name, config);

            if(string.IsNullOrEmpty(ApplicationName))
            {
                _applicationName = GetConfigValue(config["applicationName"],
                    System.Web.Hosting.HostingEnvironment.ApplicationVirtualPath);
            }

            // This is a non-standard helper setting.
            _providerName = GetConfigValue(config["providerName"], name);

            _requiresUniqueEmail = Convert.ToBoolean(GetConfigValue(config["requiresUniqueEmail"], "true"));
                _requiresQuestionAndAnswer = Convert.ToBoolean(GetConfigValue(config["requiresQuestionAndAnswer"], "true"));
                _minRequiredPasswordLength = Convert.ToInt32(GetConfigValue(config["minRequiredPasswordLength"], "5"));
                _minRequiredNonAlphanumericCharacters = Convert.ToInt32(GetConfigValue(config["minRequiredNonAlphanumericCharacters"],
                "0"));
            _enablePasswordReset = Convert.ToBoolean(GetConfigValue(config["enablePasswordReset"], "true"));
            _enablePasswordRetrieval = Convert.ToBoolean(GetConfigValue(config["enablePasswordRetrieval"], "false"));
            _passwordAttemptWindow = Convert.ToInt32(GetConfigValue(config["passwordAttemptWindow"], "10"));
            _passwordStrengthRegularExpression = GetConfigValue(config["passwordStrengthRegularExpression"], "");
            _maxInvalidPasswordAttempts = Convert.ToInt32(GetConfigValue(config["maxInvalidPasswordAttempts"],

                "5"));

            string passFormat = config["passwordFormat"];

            // If no format is specified, the default format will be hashed.
            if (passFormat == null)
                passFormat = "hashed";

            switch (passFormat.ToLower())
            {
                case "hashed":
                    _passwordFormat = MembershipPasswordFormat.Hashed;
                    break;
                case "encrypted":
                    _passwordFormat = MembershipPasswordFormat.Hashed;
                    break;
                case "clear":
                    _passwordFormat = MembershipPasswordFormat.Clear;
                    break;
                default:
                    throw new ProviderException("Password format '" + passFormat + "' is not supported. Check your web.config file.");
            }

            Configuration cfg = WebConfigurationManager.OpenWebConfiguration(System.Web.Hosting.HostingEnvironment.ApplicationVirtualPath);

            machineKey = (MachineKeySection)cfg.GetSection("system.web/machineKey");

            if (machineKey.ValidationKey.Contains("AutoGenerate"))
                if (PasswordFormat != MembershipPasswordFormat.Clear)
                    throw new ProviderException("Hashed or Encrypted passwords cannot be used with auto-generated keys.");

            MembershipSection membership = (MembershipSection)cfg.GetSection("system.web/membership");
            _userIsOnlineTimeWindow = membership.UserIsOnlineTimeWindow;
        }

    //    public string ResetPassword(string applicationName, string userName, string answer)
    //    {
    //        try
    //        {
    //            MembershipUser _mu = base.GetUser(userName);
    //            return _mu.ResetPassword(answer);
    //        }
    //        catch (Exception ex)
    //        {
    //            throw new Exception(string.Format("{0}(): Exception occurred {1}. ", this.GetType().ToString(), ex.Message));
    //        }
    //    }

        public bool UnlockUser(string applicationName, string userName)
        {
            try
            {
                MembershipUser _mu = base.GetUser(userName, false);
                return _mu.UnlockUser();
            }
            catch (Exception ex)
            {
                throw new Exception(string.Format("{0}(): Exception occurred {1}. ", this.GetType().ToString(), ex.Message));
            }
        }

        public void UpdateUser(string applicationName, MembershipUser user)
        {
            base.ApplicationName = applicationName;
            UpdateUser(user);
        }


        public bool ValidateUser(string applicationName, string username, string password)
        {
            try
            {
                return base.ValidateUser(username, password);
            }
            catch (Exception ex)
            {
                throw new Exception(string.Format("{0}(): Exception occurred {1}. ", this.GetType().ToString(), ex.Message));
            }
        }

    //    #endregion

    //    /// <summary>
    //    /// Used in the initializtion, key in web.config or the default setting if null.
    //    /// </summary>
        private string GetConfigValue(string configValue, string defaultValue)
        {
            if (String.IsNullOrEmpty(configValue))
                return defaultValue;

            return configValue;
        }



    //    public override bool ChangePassword(string username, string oldPassword, string newPassword)
    //    {
    //        throw new NotImplementedException();
    //    }

    //    public override bool ChangePasswordQuestionAndAnswer(string username, string password, string newPasswordQuestion, string newPasswordAnswer)
    //    {
    //        throw new NotImplementedException();
    //    }

    //    public override MembershipUser CreateUser(string username, string password, string email, string passwordQuestion, string passwordAnswer, bool isApproved, object providerUserKey, out MembershipCreateStatus status)
    //    {
    //        throw new NotImplementedException();
    //    }

    //    public override bool DeleteUser(string username, bool deleteAllRelatedData)
    //    {
    //        throw new NotImplementedException();
    //    }

    //    public override MembershipUserCollection FindUsersByEmail(string emailToMatch, int pageIndex, int pageSize, out int totalRecords)
    //    {
    //        throw new NotImplementedException();
    //    }

    //    public override MembershipUserCollection FindUsersByName(string usernameToMatch, int pageIndex, int pageSize, out int totalRecords)
    //    {
    //        throw new NotImplementedException();
    //    }

    //    public override MembershipUserCollection GetAllUsers(int pageIndex, int pageSize, out int totalRecords)
    //    {
    //        throw new NotImplementedException();
    //    }

    //    public override int GetNumberOfUsersOnline()
    //    {
    //        throw new NotImplementedException();
    //    }

    //    public override string GetPassword(string username, string answer)
    //    {
    //        throw new NotImplementedException();
    //    }

    //    public override MembershipUser GetUser(string username, bool userIsOnline)
    //    {
    //        throw new NotImplementedException();
    //    }

    //    public override string GetUserNameByEmail(string email)
    //    {
    //        try
    //        {
    //            return base.GetUserNameByEmail(email);
    //        }
    //        catch (Exception ex)
    //        {
    //            throw new Exception(string.Format("{0}(): Exception occurred {1}. ", this.GetType().ToString(), ex.Message));
    //        }
    //    }

        int _minRequiredNonAlphanumericCharacters;
        public override int MinRequiredNonAlphanumericCharacters
        {
            get
            {
                return _minRequiredNonAlphanumericCharacters;
            }
        }

        int _minRequiredPasswordLength;
        public override int MinRequiredPasswordLength { get { return _minRequiredPasswordLength; } }

        bool _enablePasswordReset;
        public override bool EnablePasswordReset { get { return _enablePasswordReset; } }

        bool _enablePasswordRetrieval;
        public override bool EnablePasswordRetrieval { get { return _enablePasswordRetrieval; } }

        int _passwordAttemptWindow;
        public override int PasswordAttemptWindow { get { return _passwordAttemptWindow; } }

        string _passwordStrengthRegularExpression;
        public override string PasswordStrengthRegularExpression { get { return _passwordStrengthRegularExpression; } }

        MembershipPasswordFormat _passwordFormat;
        public override MembershipPasswordFormat PasswordFormat { get { return _passwordFormat; } }

        int _maxInvalidPasswordAttempts;
        public override int MaxInvalidPasswordAttempts { get { return _maxInvalidPasswordAttempts; } }

        bool _requiresQuestionAndAnswer;
        public override bool RequiresQuestionAndAnswer { get { return _requiresQuestionAndAnswer; } }

        bool _requiresUniqueEmail;
        public override bool RequiresUniqueEmail { get { return _requiresUniqueEmail; } }

        public override string ResetPassword(string username, string answer)
        {
            throw new NotImplementedException();
        }

        public override bool UnlockUser(string userName)
        {
            throw new NotImplementedException();
        }


        public override void UpdateUser(MembershipUser user)
        {
            try
            {
                base.UpdateUser(user);
            }
            catch (Exception ex)
            {
                throw new Exception(string.Format("{0}(): Exception occurred {1}. ", this.GetType().ToString(), ex.Message));
            }
        }

        public override bool ValidateUser(string username, string password)
        {
            return base.ValidateUser(username, password);

        }

    //    public override MembershipUser GetUser(object providerUserKey, bool userIsOnline)
    //    {
    //        throw new NotImplementedException();
    //    }
    }
}
