﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using NST.Auth.ProviderContracts;
using NST.Auth.HelperLib;
using System.ServiceModel;
using NST.Auth.DataContracts;
using System.Data.Linq;
using System.Web.Security;
using System.Web.ClientServices.Providers;

namespace NST.Auth.AspNetDb
{
    public class AuthenticatorProvider : IAuthenticatorProvider
    {
        string _aspNetDBCStr;

        public AuthenticatorProvider()
        {
            _aspNetDBCStr = Helpers.CreateSqlAuthConnectionString(CachedCredentials.UserName, CachedCredentials.Password);
        }

        public MembershipCreateStatus AddUpdateUser(UserInfoDC obj)
        {
            aspnet_User _dalObj = null;
            DataLoadOptions _options = new DataLoadOptions();
            bool _updateSuccessful = false;
            MembershipCreateStatus _status = MembershipCreateStatus.UserRejected;
            bool _isNewUser = !obj.Id.HasValue;

            try
            {

                if (obj == null || string.IsNullOrEmpty(Membership.ApplicationName))
                {
                    throw new Exception("CreateUpdateUser requires an AuthUser & ApplicationName must be valid.");
                }

                // If user is a member of the required admin role for authenticator, return all applications
                using (AspNetDBDataContext db = new AspNetDBDataContext(_aspNetDBCStr))
                {
                    db.DeferredLoadingEnabled = false;
                    _options.LoadWith<aspnet_User>(a => a.aspnet_Membership);
                    _options.LoadWith<aspnet_User>(a => a.aspnet_Application);

                    db.LoadOptions = _options;

                    if (obj == null || obj.ApplicationId == null)
                    {
                        throw new Exception("CreateUpdateUser requires an AuthUser & ApplicationId must be valid.");
                    }

                    // Otherwise it's an New users
                    // If a consumer passes in an ApplicationName or ApplicationId, use it instead of
                    // what's defined in the web.config
                    if (!string.IsNullOrEmpty(obj.ApplicationName))
                        Membership.ApplicationName = obj.ApplicationName;
                    else if(obj.ApplicationId.HasValue)
                        Membership.ApplicationName = (from o in db.aspnet_Applications.Where(a => a.ApplicationId == obj.ApplicationId.Value)
                                        select o.ApplicationName).First();

                    // If an ID was passed in, then it's an update
                    if (obj.Id.HasValue)
                    {
                        // If a user passes in app name and no Id.
                        if(!obj.ApplicationId.HasValue && !string.IsNullOrEmpty(obj.ApplicationName))
                            obj.ApplicationId = (from o in db.aspnet_Applications.Where(a => a.ApplicationId == obj.ApplicationId.Value)
                                                         select o.ApplicationId).First();

                        _dalObj = (from o in db.aspnet_Users.Where(o => o.ApplicationId == obj.ApplicationId && o.UserId == obj.Id.Value)
                                   select o).First();

                        if (_dalObj == null)
                            throw new Exception(string.Format("{0}(): No application found for Id {1}.", this.GetType().ToString(), obj.Id.Value));

                    }
                    else
                    {



                        var _user = Membership.CreateUser(obj.Name, obj.Password, obj.Email, obj.PasswordQuestion, obj.PasswordAnswer, obj.IsApproved, null, out _status);

                        if (_user != null && _status == MembershipCreateStatus.Success)
                        {
                            _dalObj = (from o in db.aspnet_Users.Where(o => o.aspnet_Application.ApplicationName == Membership.ApplicationName && o.UserName == _user.UserName)
                                       select o).First();
                        }
                    }

                    if (_dalObj != null)
                    {
                        _dalObj.UserName = obj.Name;
                        _dalObj.LoweredUserName = obj.Name.ToLower();
                        _dalObj.MobileAlias = obj.MobileAlias;
                        _dalObj.IsAnonymous = obj.IsAnonymous;
                        _dalObj.aspnet_Membership.Email = obj.Email;
                        _dalObj.aspnet_Membership.LoweredEmail = obj.Email.ToLower();
                        _dalObj.aspnet_Membership.IsApproved = obj.IsApproved;
                        _dalObj.aspnet_Membership.IsLockedOut = obj.IsLockedOut;
                        _dalObj.aspnet_Membership.Comment = obj.Comment;

                        db.SubmitChanges();

                        _status = MembershipCreateStatus.Success;

                        // Here you need to make sure we're only trying to update the password if that's what somebody was trying to do.
                        // Also, that a password change actually works.  
                        if (!_isNewUser && (obj.Password != _dalObj.aspnet_Membership.Password || obj.PasswordAnswer != _dalObj.aspnet_Membership.PasswordAnswer || 
                            obj.PasswordQuestion != _dalObj.aspnet_Membership.PasswordQuestion))
                        {
                            try
                            {
                                MembershipUser _user = Membership.GetUser(_dalObj.UserName);
                                
                                if (!string.IsNullOrEmpty(obj.PasswordQuestion))
                                    _updateSuccessful = _user.ChangePasswordQuestionAndAnswer(obj.Password, obj.PasswordQuestion, obj.PasswordAnswer);
                                else
                                {
                                    _updateSuccessful = _user.ChangePassword(_dalObj.aspnet_Membership.Password, obj.Password);
                                }

                                if (!_updateSuccessful)
                                    _status = MembershipCreateStatus.InvalidPassword;
                            }
                            catch (Exception ex)
                            {
                                throw new Exception(string.Format("{0}(): Exception occurred {1}. ", this.GetType().ToString(), ex.Message));
                            }
                        }

                        // Update Roles
                        // First clear all roles for the User and then repopulate
                        //db.ExecuteCommand(string.Format("Delete from 
                        //ClientRoleProvider _roleProvider = new ClientRoleProvider();

                        //SqlRoleProvider _roleProvider = new SqlRoleProvider();
                        //_roleProvider.ApplicationName = Membership.ApplicationName;
                        string[] _userName = new string[1];
                        _userName[0] = _dalObj.UserName;
                        Roles.ApplicationName = Membership.ApplicationName;

                        if(!_isNewUser)
                        {
                            //var _roles = _roleProvider.GetRolesForUser(_dalObj.UserName);
                            var _roles = Roles.GetRolesForUser(_dalObj.UserName);

                            if (_roles != null && _roles.Count() > 0)
                            {
                                Roles.RemoveUsersFromRoles(_userName, _roles);
                            }
                        }

                        List<string> _roleNames = new List<string>();

                        if (obj.Roles != null)
                        {
                            foreach (var r in obj.Roles)
                            {
                                _roleNames.Add(r.Name);
                            }

                            if (_roleNames.Count > 0)
                            {
                                Roles.AddUsersToRoles(_userName, _roleNames.ToArray());
                            }
                        }

                    }
                }
            }
            catch (Exception ex)
            {
                _status = MembershipCreateStatus.UserRejected;
                throw new Exception(string.Format("{0}(): Exception occurred {1}. ", this.GetType().ToString(), ex.Message));
                
            }

            return _status;
        }

        public Guid CreateUpdateApplication(ApplicationDC obj)
        {
            Guid _appId = Guid.Empty;
            DataLoadOptions _options = new DataLoadOptions();
            bool _isNew = true;
            aspnet_Application _dalObj = new aspnet_Application();

            try
            {
                // If user is a member of the required admin role for authenticator, return all applications
                using (AspNetDBDataContext db = new AspNetDBDataContext(_aspNetDBCStr))
                {
                    db.DeferredLoadingEnabled = false;
                    db.LoadOptions = _options;

                    if (obj == null)
                    {
                        return _appId;
                    }

                    if (obj.Id.HasValue)
                    {
                        _dalObj = (from o in db.aspnet_Applications.Where(o => o.ApplicationId == obj.Id.Value)
                                   select o).First();

                        if (_dalObj == null)
                            throw new Exception(string.Format("{0}(): No application found for Id {1}.", this.GetType().ToString(), obj.Id.Value));

                        _isNew = false;
                    }
                    else
                    {
                        var _objs = from o in db.aspnet_Applications.Where(o => o.LoweredApplicationName == obj.Name.ToLower())
                                   select o;
                        // Make sure there isn't an existing app name.
                        if (_objs != null && _objs.Count() > 0)
                            throw new Exception(string.Format("{0} Application Name {1} already exists.", this.GetType().ToString(), obj.Name));

                    }

                    _dalObj.ApplicationName = obj.Name;
                    _dalObj.Description = obj.Description;
                    _dalObj.LoweredApplicationName = obj.Name.ToLower();

                    if (_isNew)
                    {
                        db.aspnet_Applications.InsertOnSubmit(_dalObj);
                    }

                    db.SubmitChanges();

                    _appId = _dalObj.ApplicationId;
                }
            }
            catch (Exception ex)
            {
                throw new Exception(string.Format("{0}(): Exception occurred {1}. ", this.GetType().ToString(), ex.Message));
            }

            return _appId;
        }

        public Guid CreateUpdateRole(RoleDC obj)
        {
            aspnet_Role _dalObj = new aspnet_Role();
            DataLoadOptions _options = new DataLoadOptions();
            Guid _roleId = Guid.Empty;

            bool _isNew = true;

            try
            {
                // If user is a member of the required admin role for authenticator, return all applications
                using (AspNetDBDataContext db = new AspNetDBDataContext(_aspNetDBCStr))
                {
                    db.DeferredLoadingEnabled = false;
                    db.LoadOptions = _options;

                    if (obj == null && !obj.ApplicationId.HasValue)
                    {
                        throw new Exception("CreateUpdateRole requires an AuthRole & ApplicationId must be valid.");
                    }

                    if (obj.Id.HasValue)
                    {
                        _dalObj = (from o in db.aspnet_Roles.Where(o => o.ApplicationId == obj.ApplicationId && o.RoleId == obj.Id.Value)
                                   select o).First();

                        if (_dalObj == null)
                            throw new Exception(string.Format("{0}(): No application found for Id {1}.", this.GetType().ToString(), obj.Id.Value));

                        _isNew = false;
                    }

                    _dalObj.ApplicationId = obj.ApplicationId.Value;
                    _dalObj.RoleName = obj.Name;
                    _dalObj.Description = obj.Description;
                    _dalObj.LoweredRoleName = obj.Name.ToLower();

                    if (_isNew)
                    {
                        db.aspnet_Roles.InsertOnSubmit(_dalObj);
                    }

                    db.SubmitChanges();

                    _roleId = _dalObj.RoleId;
                }
            }
            catch (Exception ex)
            {
                throw new Exception(string.Format("{0}(): Exception occurred {1}. ", this.GetType().ToString(), ex.Message));
            }

            return _roleId;
        }

        public MembershipCreateStatus CreateUser(NewUserDC obj)
        {
            MembershipCreateStatus _status = MembershipCreateStatus.ProviderError;
            if (!string.IsNullOrEmpty(obj.ApplicationName))
                Membership.ApplicationName = obj.ApplicationName;

            var _user = Membership.CreateUser(obj.UserName, obj.Password, obj.Email, obj.PasswordQuestion, obj.PasswordAnswer, obj.IsApproved, null, out _status);

            return _status;
        }

        public void DeleteApplications(IList<string> applicationNames)
        {
            try
            {
                // If user is a member of the required admin role for authenticator, return all applications
                using (AspNetDBDataContext db = new AspNetDBDataContext(_aspNetDBCStr))
                {
                    var _dalObj = from o in db.aspnet_Applications.Where(o => applicationNames.AsEnumerable().Contains(o.ApplicationName))
                                   select o;

                    db.aspnet_Applications.DeleteAllOnSubmit(_dalObj);
                    db.SubmitChanges();
                }
            }
            catch (Exception ex)
            {
                throw new Exception(string.Format("{0}(): Exception occurred {1}. ", this.GetType().ToString(), ex.Message));
            }
        }

        public void DeleteRoles(string applicationName, IList<string> roleNames)
        {
            try
            {
                // Need to use RoleProvider here
                //AuthMembershipProvider _prvdr = new AuthMembershipProvider(applicationName);
                
                 //foreach (var r in roleNames)
                 //   {
                 //       Roles.DeleteRole(r);
                 //   }

                // Cascade should be set for relationships with Roles.
                using (AspNetDBDataContext db = new AspNetDBDataContext(_aspNetDBCStr))
                {
                    var _dalObj = from o in db.aspnet_Roles.Where(o => o.aspnet_Application.ApplicationName == applicationName && roleNames.AsEnumerable().Contains(o.RoleName))
                                  select o;

                    db.aspnet_Roles.DeleteAllOnSubmit(_dalObj);
                    db.SubmitChanges();
                }
            }
            catch (Exception ex)
            {
                throw new Exception(string.Format("{0}(): Exception occurred {1}. ", this.GetType().ToString(), ex.Message));
            }
        }

        public void DeleteUsers(string applicationName, IList<string> userNames)
        {            
            try
            {
                //AuthMembershipProvider _prvd = new AuthMembershipProvider(applicationName);
                if (!string.IsNullOrEmpty(applicationName))
                    Membership.ApplicationName = applicationName;

                foreach (var userName in userNames)
                {
                    bool _success = Membership.DeleteUser(userName, true);
                }

                // If user is a member of the required admin role for authenticator, return all applications
                //using (AspNetDBDataContext db = new AspNetDBDataContext(_aspNetDBCStr))
                //{       
                    //var _dalObj = from o in db.aspnet_Users.Where(o => o.ApplicationId == applicationId && userIds.AsEnumerable().Contains(o.UserId))
                    //              select o;

                    //db.aspnet_Users.DeleteAllOnSubmit(_dalObj);
                    //db.SubmitChanges();
                //}
            }
            catch (Exception ex)
            {
                throw new Exception(string.Format("{0}(): Exception occurred {1}. ", this.GetType().ToString(), ex.Message));
            }
        }

        public IList<ApplicationDC> GetApplications()
        {
            List<ApplicationDC> _objs = new List<ApplicationDC>();

            DataLoadOptions options = new DataLoadOptions();

            try
            {
                using (AspNetDBDataContext db = new AspNetDBDataContext(_aspNetDBCStr))
                {
                    db.DeferredLoadingEnabled = false;
                    db.LoadOptions = options;

                    _objs = (from c in db.aspnet_Applications
                             select new ApplicationDC
                             {
                                 Id = c.ApplicationId,
                                 Name = c.ApplicationName,
                                 Description = c.Description,
                                 ApplicationId = c.ApplicationId
                             }).ToList();
                }

            }
            catch (Exception ex)
            {
                throw new Exception(string.Format("{0}(): Exception occurred {1}. ", this.GetType().ToString(), ex.Message));
            }


            return _objs;
        }

        public IList<RoleDC> GetRoles(IList<string> roleNames, string appName, Guid? appId, string userName)
        {
            List<RoleDC> _objs = new List<RoleDC>();

            try
            {
                DataLoadOptions options = new DataLoadOptions();

                var predicate = PredicateBuilder.True<aspnet_Role>();

                if (roleNames != null && roleNames.Count > 0)
                {
                    predicate = predicate.And(p => roleNames.AsEnumerable().Contains(p.RoleName));
                }
                else
                {
                    if (appId.HasValue)
                    {
                        predicate = predicate.And(p => p.aspnet_Application.ApplicationId == appId.Value);
                    }
                    else if(!string.IsNullOrEmpty(appName))
                    {
                        predicate = predicate.And(p => p.aspnet_Application.ApplicationName == appName);
                    }
                    else
                        throw new Exception(string.Format("{0} must contain a Role name or ApplicationName or ApplicationId", this.GetType().ToString()));

                    if (!string.IsNullOrEmpty(userName))
                    {
                        predicate = predicate.And(p => p.aspnet_UsersInRoles.Any(a => a.aspnet_User.UserName == userName));
                    }
                }

                using (AspNetDBDataContext db = new AspNetDBDataContext(_aspNetDBCStr))
                {
                    db.DeferredLoadingEnabled = false;
                    db.LoadOptions = options;

                    _objs = (from o in db.aspnet_Roles.Where(predicate)
                                       orderby o.RoleName
                                       select new RoleDC
                                       {
                                           Id = o.RoleId,
                                           Name = o.RoleName,
                                           Description = o.Description,
                                           ApplicationId = o.ApplicationId
                                       }).ToList();
                }
            }
            catch (Exception ex)
            {
                throw new Exception(string.Format("{0}(): Exception occurred {1}. ", this.GetType().ToString(), ex.Message));
            }

            return _objs;
        }

        public IList<UserInfoDC> GetUsers(string appName, string roleName)
        {
            List<UserInfoDC> _objs = new List<UserInfoDC>();

            try
            {
                DataLoadOptions _options = new DataLoadOptions();

                var predicate = PredicateBuilder.True<aspnet_User>();

                if (!string.IsNullOrEmpty(appName))
                {
                    predicate = predicate.And(p => p.aspnet_Application.ApplicationName == appName);
                }

                if (!string.IsNullOrEmpty(roleName))
                {
                    predicate = predicate.And(p => p.aspnet_UsersInRoles.Any(u => u.aspnet_Role.RoleName == roleName));
                }

                using (AspNetDBDataContext db = new AspNetDBDataContext(_aspNetDBCStr))
                {
                    db.DeferredLoadingEnabled = false;
                    _options.LoadWith<aspnet_User>(a => a.aspnet_Membership);
                    _options.LoadWith<aspnet_User>(a => a.aspnet_UsersInRoles);
                    _options.LoadWith<aspnet_UsersInRole>(a => a.aspnet_Role);

                    db.LoadOptions = _options;

                    var _dalObjs = from o in db.aspnet_Users.Where(predicate)
                                       orderby o.UserName
                                       select o;

                    _objs.AddRange(UserInfoAssembler<UserInfoDC>.AssembleContracts(_dalObjs));
                }
            }
            catch (Exception ex)
            {
                throw new Exception(string.Format("{0}(): Exception occurred {1}. ", this.GetType().ToString(), ex.Message));
            }

            return _objs;
        }

     

    }
}
