﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web.Security;
using System.Diagnostics;
using System.Reflection;
using System.Web.Hosting;
using System.Configuration.Provider;
using System.Web.Configuration;
using System.Configuration;
using System.Collections.Specialized;

namespace NST.Auth.AspCredentials
{
    public class AuthMemberShipProvider : MembershipProvider
    {
        private MachineKeySection machineKey;

        #region Private Fields

        string _providerName;

        private TimeSpan _userIsOnlineTimeWindow;
        public TimeSpan UserIsOnlineTimeWindow
        {
            get { return _userIsOnlineTimeWindow; }
        }

	    #endregion
        
        
        string _applicationName;
        public override string ApplicationName
        {
            get
            {
                return _applicationName;
            }
            set
            {
                _applicationName = value;
            }
        }

        public override bool ChangePassword(string userName, string oldPassword, string newPassword)
        {
            MembershipUser _membershipUser = Membership.GetUser(userName);
            return _membershipUser.ChangePassword(_membershipUser.GetPassword(), newPassword);            
        }

        public override bool ChangePasswordQuestionAndAnswer(string userName, string password, string passwordQuestion, string passwordAnswer)
        {
            MembershipUser membershipUser = Membership.GetUser(userName);
            return membershipUser.ChangePasswordQuestionAndAnswer(password, passwordQuestion, passwordAnswer);
        }

        public override MembershipUser CreateUser(string username, string password, string email, string passwordQuestion, string passwordAnswer, bool isApproved, object providerUserKey, out MembershipCreateStatus status)
        {
            try
            {
                 return Membership.CreateUser(username, passwordAnswer, email, passwordQuestion, passwordAnswer, isApproved, providerUserKey, out status);
            }
            catch (Exception ex)
            {
                throw new Exception(string.Format("{0}(): Exception occurred {1}. ", GetMethodName(), ex.Message));
            }
        }

        public override bool DeleteUser(string username, bool deleteAllRelatedData)
        {
            try
            {
                return Membership.DeleteUser(username, deleteAllRelatedData);
            }
            catch (Exception ex)
            {
                throw new Exception(string.Format("{0}(): Exception occurred {1}. ", GetMethodName(), ex.Message));
            }
        }

        bool _enablePasswordReset;
        public override bool EnablePasswordReset
        {
            get { return _enablePasswordReset; }
        }

        bool _enablePasswordRetrieval;
        public override bool EnablePasswordRetrieval
        {
            get { return _enablePasswordRetrieval; }
        }

        public override MembershipUserCollection FindUsersByEmail(string emailToMatch, int pageIndex, int pageSize, out int totalRecords)
        {
            try
            {
                return Membership.FindUsersByEmail(emailToMatch, pageIndex, pageSize, out totalRecords);
            }
            catch (Exception ex)
            {
                throw new Exception(string.Format("{0}(): Exception occurred {1}. ", GetMethodName(), ex.Message));
            }
        }

        public override MembershipUserCollection FindUsersByName(string usernameToMatch, int pageIndex, int pageSize, out int totalRecords)
        {
            try
            {
                return Membership.FindUsersByName(usernameToMatch, pageIndex, pageSize, out totalRecords);
            }
            catch (Exception ex)
            {
                throw new Exception(string.Format("{0}(): Exception occurred {1}. ", GetMethodName(), ex.Message));
            }
        }

        public override MembershipUserCollection GetAllUsers(int pageIndex, int pageSize, out int totalRecords)
        {
            try
            {
                return Membership.GetAllUsers(pageIndex, pageSize, out totalRecords);
            }
            catch (Exception ex)
            {
                throw new Exception(string.Format("{0}(): Exception occurred {1}. ", GetMethodName(), ex.Message));
            }
        }
                
        public override int GetNumberOfUsersOnline()
        {
            try
            {
                return Membership.GetNumberOfUsersOnline();
            }
            catch (Exception ex)
            {
                throw new Exception(string.Format("{0}(): Exception occurred {1}. ", GetMethodName(), ex.Message));
            }
        }

        public override string GetPassword(string userName, string answer)
        {
            try
            {
                MembershipUser _membershipUser = Membership.GetUser(userName);
                return _membershipUser.GetPassword(answer);
            }
            catch (Exception ex)
            {
                throw new Exception(string.Format("{0}(): Exception occurred {1}. ", GetMethodName(), ex.Message));
            }
        }

        public override MembershipUser GetUser(string userName, bool userIsOnline)
        {
            try
            {
                return Membership.GetUser(userName, userIsOnline);
            }
            catch (Exception ex)
            {
                throw new Exception(string.Format("{0}(): Exception occurred {1}. ", GetMethodName(), ex.Message));
            }
        }

        public override MembershipUser GetUser(object providerUserKey, bool userIsOnline)
        {
            try
            {
                return Membership.GetUser(providerUserKey, userIsOnline);
            }
            catch (Exception ex)
            {
                throw new Exception(string.Format("{0}(): Exception occurred {1}. ", GetMethodName(), ex.Message));
            }
        }

        public override string GetUserNameByEmail(string email)
        {
            try
            {
                return Membership.GetUserNameByEmail(email);
            }
            catch (Exception ex)
            {
                throw new Exception(string.Format("{0}(): Exception occurred {1}. ", GetMethodName(), ex.Message));
            }
        }

        public override void Initialize(string name, NameValueCollection config)  
        {  

             if (config == null)  
                throw new ArgumentNullException("config");  

             if (name == null || name.Length == 0)  
                 name = "OsirisMembershipProvider";  

             if (String.IsNullOrEmpty(config["description"]))  
             {  

                 config.Remove("description");  

                 config.Add("description", "Osiris Membership Provider");  

              }  

             // Initialize base class  

             base.Initialize(name, config);      

             _applicationName = GetConfigValue(config["applicationName"],  
                 System.Web.Hosting.HostingEnvironment.ApplicationVirtualPath);      

             // This is a non-standard helper setting.  
             _providerName = GetConfigValue(config["providerName"], name);  
    

             // Sets the default parameters for all the Membership Provider settings      

             _requiresUniqueEmail = Convert.ToBoolean(GetConfigValue(config["requiresUniqueEmail"], "true"));  
             _requiresQuestionAndAnswer = Convert.ToBoolean(GetConfigValue(config["requiresQuestionAndAnswer"], "true"));  
             _minRequiredPasswordLength = Convert.ToInt32(GetConfigValue(config["minRequiredPasswordLength"], "5"));  
             _minRequiredNonAlphanumericCharacters = Convert.ToInt32(GetConfigValue(config["minRequiredNonAlphanumericCharacters"], "0"));  
             _enablePasswordReset = Convert.ToBoolean(GetConfigValue(config["enablePasswordReset"], "true"));  
             _enablePasswordRetrieval = Convert.ToBoolean(GetConfigValue(config["enablePasswordRetrieval"], "false"));  
            _passwordAttemptWindow = Convert.ToInt32(GetConfigValue(config["passwordAttemptWindow"], "10"));  
             _passwordStrengthRegularExpression = GetConfigValue(config["passwordStrengthRegularExpression"], "");  
             _maxInvalidPasswordAttempts = Convert.ToInt32(GetConfigValue(config["maxInvalidPasswordAttempts"], "5"));     

             string passFormat = config["passwordFormat"];  

    

             // If no format is specified, the default format will be hashed.  

             if (passFormat == null)  
                 passFormat = "hashed";      

             switch (passFormat.ToLower())  
             {  
                 case "hashed":  
                     _passwordFormat = MembershipPasswordFormat.Hashed;  
                     break;  

                 case "encrypted":  
                     _passwordFormat = MembershipPasswordFormat.Hashed;  
                     break;  

                 case "clear":  
                     _passwordFormat = MembershipPasswordFormat.Clear;  
                     break;  

                 default:  
                     throw new ProviderException("Password format '" + passFormat + "' is not supported. Check your web.config file.");  

             }  

    

             Configuration cfg = WebConfigurationManager.OpenWebConfiguration(HostingEnvironment.ApplicationVirtualPath);      

             machineKey = (MachineKeySection)cfg.GetSection("system.web/machineKey");      

             if (machineKey.ValidationKey.Contains("AutoGenerate"))  
                 if (PasswordFormat != MembershipPasswordFormat.Clear)  
                     throw new ProviderException("Hashed or Encrypted passwords cannot be used with auto-generated keys.");  

             MembershipSection membership = (MembershipSection)cfg.GetSection("system.web/membership");  

             _userIsOnlineTimeWindow = membership.UserIsOnlineTimeWindow;  
        }

        int _maxInvalidPasswordAttempts;
        public override int MaxInvalidPasswordAttempts
        {
            get { return _maxInvalidPasswordAttempts; }
            
        }

        int _minRequiredNonAlphanumericCharacters;
        public override int MinRequiredNonAlphanumericCharacters
        {
            get { return _minRequiredNonAlphanumericCharacters; }
        }

        int _minRequiredPasswordLength;
        public override int MinRequiredPasswordLength
        {
            get { return Membership.MinRequiredPasswordLength; }
        }

        int _passwordAttemptWindow;
        public override int PasswordAttemptWindow
        {
            get { return _passwordAttemptWindow; }
        }

        MembershipPasswordFormat _passwordFormat;
        public override MembershipPasswordFormat PasswordFormat
        {
            get { return _passwordFormat; }
        }

        string _passwordStrengthRegularExpression;
        public override string PasswordStrengthRegularExpression
        {
            get { return _passwordStrengthRegularExpression; }
        }

        bool _requiresQuestionAndAnswer;
        public override bool RequiresQuestionAndAnswer
        {
           get { return _requiresQuestionAndAnswer; }
        }

        bool _requiresUniqueEmail;
        public override bool RequiresUniqueEmail
        {
            get
            {
                return _requiresUniqueEmail;
            }
        }

        public override string ResetPassword(string username, string answer)
        {
            try
            {
                MembershipUser _u = Membership.GetUser(username);
                return _u.ResetPassword(answer);
                
            }
            catch (Exception ex)
            {
                throw new Exception(string.Format("{0}(): Exception occurred {1}. ", GetMethodName(), ex.Message));
            }
        }

        public override bool UnlockUser(string userName)
        {
            try
            {
                MembershipUser _u = Membership.GetUser(userName);
                return _u.UnlockUser();
            }
            catch (Exception ex)
            {
                throw new Exception(string.Format("{0}(): Exception occurred {1}. ", GetMethodName(), ex.Message));
            }
        }

        public override void UpdateUser(MembershipUser user)
        {
            try
            {
                Membership.UpdateUser(user);
            }
            catch (Exception ex)
            {
                throw new Exception(string.Format("{0}(): Exception occurred {1}. ", GetMethodName(), ex.Message));
            }
        }

        public override bool ValidateUser(string username, string password)
        {
            try
            {
                return Membership.ValidateUser(username, password);
            }
            catch (Exception ex)
            {
                throw new Exception(string.Format("{0}(): Exception occurred {1}. ", GetMethodName(), ex.Message));
            }
        }

        #region Private Methods
	    /// <summary>  
        /// Used in the initializtion, key in web.config or the default setting if null.  
       /// </summary>  
        private string GetConfigValue(string configValue, string defaultValue)  
         {  
             if (String.IsNullOrEmpty(configValue))  
                 return defaultValue;     

             return configValue;  
         }  
	 
        private string GetMethodName()
        {
            StackTrace st = new StackTrace();
            StackFrame sf = st.GetFrame(1);
            MethodBase mb = sf.GetMethod();

            return mb.Name;
        }
    
	    #endregion

    }
}
