﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Reflection;

namespace NST.Auth.ProviderContracts
{
    /// <summary>
    ///  T also for any type of interface to be passed in and as long as what is specified in the assemblyName and className instantiates
    ///  a Type that implements T.
    /// </summary>
    /// <typeparam name="T"></typeparam>
    public static class ProviderFactory<T>
    {
        private static volatile Dictionary<string, Dictionary<string, T>> _providerCache = new Dictionary<string, Dictionary<string, T>>();
        static object _lock = new object();

        #region Private Fields
        #endregion

        public static T GetProvider(string assemblyName, string className)
        {
            T _provider = default(T);

            Dictionary<string, T> _existingAssembly = new Dictionary<string, T>();

            // Check to see if this assembly has been added to the dictionary before
            lock (_lock)
            {
                if (_providerCache.ContainsKey(assemblyName))
                {
                    _existingAssembly = _providerCache[assemblyName];
                    // See if className in the assembly has already been loaded, if so assign to _provider to return
                    if (_existingAssembly.ContainsKey(className))
                    {
                        _provider = _existingAssembly[className];
                    }
                    else
                    {
                        _provider = CreateProvider(assemblyName, className);

                        _existingAssembly.Add(className, _provider);
                    }

                }
                else
                {
                    if (!_existingAssembly.ContainsKey(assemblyName))
                    {
                        _provider = CreateProvider(assemblyName, className);
                        _existingAssembly.Add(className, _provider);
                        _providerCache.Add(assemblyName, _existingAssembly);
                    }
                }
            }

            return _provider;
        }


        /// <summary>
        /// Create a provider based on the assemblyName and className passed in.  Note the assembly must be in the bin or GAC.
        /// </summary>
        /// <param name="assemblyName">name of assembly to load</param>
        /// <param name="className">class within the assembly to load.</param>
        /// <returns></returns>
        private static T CreateProvider(string assemblyName, string className)
        {
            // Create provider specified in config
            Assembly assemb = Assembly.Load(new AssemblyName(assemblyName));

            if (assemb == null)
            {
                throw new Exception(string.Format("Could not create instance of {0}", assemblyName));
            }

            object providerObj = assemb.CreateInstance(className, true);
            // See if there's a problem creating the instance
            if (providerObj == null)
            {
                throw new Exception(string.Format("Could not create instance of {0}", className));
            }

            // Throw an exception if object does not implement IDataProvider
            T provider = (T)providerObj;
            if (provider == null)
            {
                throw new Exception(string.Format("{0} does not implement {1}.", assemblyName, className));
            }

            // Return the provider
            return provider;
        }
    }
}
