﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using NST.Auth.ServiceContracts;
using System.ServiceModel;
using System.ServiceModel.Activation;
using NST.Auth.ProviderContracts;
using System.Configuration;
using Logging;
using System.Security.Permissions;
using NST.Auth.MessageContracts;

namespace NST.Auth.ServiceImplementation
{
    [UseFormsAuthenticationBehaviorAttribute]
    [ServiceBehavior(Name = "AuthenticatorService", Namespace = "http://n-stech.Auth.ServiceImplementation/2010/05", InstanceContextMode = InstanceContextMode.PerSession,
            ConcurrencyMode = ConcurrencyMode.Single)]
    [AspNetCompatibilityRequirements(RequirementsMode = AspNetCompatibilityRequirementsMode.Allowed)]
    public class AuthenticatorService : IAuthenticatorService 
    {
        public static IAuthenticatorProvider _provider;
        private const int eventPriority = 500;
        private string _applicationName;
        private bool _logAuthenticationFailures;
        private bool _logAuthorizationFailures;

        private enum MethodCode
        {
            AuthenticateUser = 4000050,
            CreateUpdateApplication = 4000100,
            CreateUpdateRole = 4000150,
            CreateUser = 4000175,
            DeleteApplications = 4000200,
            GetApplications = 4000250,
            GetAuthRoles = 4000300,
            GetAuthUsers = 4000350,
            DeleteRoles = 4000400,
            DeleteUsers = 4000450,
            AddUpdateUser = 4000500
            
        }

        #region Ctor

        public AuthenticatorService()
        {
            _provider = ProviderFactory<IAuthenticatorProvider>.GetProvider(ConfigurationManager.AppSettings["AuthenticatorProviderAssembly"], ConfigurationManager.AppSettings["AuthenticatorProviderClass"]);
            _applicationName = ConfigurationManager.AppSettings["wcfAppName"];           
            
        }

        #endregion
        
        #region IAspNetRoleService Members
        
        public AddUpdateUserResponse AddUpdateUser(AddUpdateUserRequest request)
        {
            int methodCode = (int)MethodCode.AddUpdateUser;
            string methodName = Enum.GetName(typeof(MethodCode), MethodCode.AddUpdateUser);
            AddUpdateUserResponse _response = new AddUpdateUserResponse();

            try
            {
                if (OperationContext.Current.ServiceSecurityContext.PrimaryIdentity.IsAuthenticated)
                {
                    if (request != null && request.UserInfo != null)
                    {
                        _response.Status = _provider.AddUpdateUser(request.UserInfo);
                    }
                }
            }
            catch (Exception ex)
            {
                _response.Status = System.Web.Security.MembershipCreateStatus.UserRejected;
                Log.LogEvent(methodCode, eventPriority, System.Diagnostics.TraceEventType.Critical, _applicationName, methodName, string.Format("Message: {0}. InnerException: {1}.", ex.Message, ex.InnerException != null ? ex.InnerException.ToString() : string.Empty));
            }

            return _response;
        }
        
        public AuthenticateUserResponse AuthenticateUser(AuthenticateUserRequest request)
        {
            AuthenticateUserResponse _response = new AuthenticateUserResponse();

            try
            {
                _response.IsAuthenticated = OperationContext.Current.ServiceSecurityContext.PrimaryIdentity.IsAuthenticated;
            }
            catch (Exception ex)
            {
                throw new Exception(string.Format("{0}(): Exception occurred {1}. ", this.GetType().ToString(), ex.Message));
            }

            return _response;
        }

        public CreateUpdateApplicationResponse CreateUpdateApplication(CreateUpdateApplicationRequest request)
        {
            int methodCode = (int)MethodCode.CreateUpdateApplication;
            string methodName = Enum.GetName(typeof(MethodCode), MethodCode.CreateUpdateApplication);
            CreateUpdateApplicationResponse _response = new CreateUpdateApplicationResponse();

            try
            {
                if (OperationContext.Current.ServiceSecurityContext.PrimaryIdentity.IsAuthenticated)
                {
                    if (request != null && request.App != null)
                    {
                        _response.AppId = _provider.CreateUpdateApplication(request.App);
                    }
                }
            }
            catch (Exception ex)
            {
                Log.LogEvent(methodCode, eventPriority, System.Diagnostics.TraceEventType.Critical, _applicationName, methodName, string.Format("Message: {0}. InnerException: {1}.", ex.Message, ex.InnerException != null ? ex.InnerException.ToString() : string.Empty));
            }

            return _response;
        }

        public CreateUpdateRoleResponse CreateUpdateRole(CreateUpdateRoleRequest request)
        {
            int methodCode = (int)MethodCode.CreateUpdateRole;
            string methodName = Enum.GetName(typeof(MethodCode), MethodCode.CreateUpdateRole);
            CreateUpdateRoleResponse _response = new CreateUpdateRoleResponse();

            try
            {
                if (OperationContext.Current.ServiceSecurityContext.PrimaryIdentity.IsAuthenticated)
                {
                    if (request != null && request.Role != null)
                    {
                        _response.RoleId = _provider.CreateUpdateRole(request.Role);
                    }
                }
            }
            catch (Exception ex)
            {
                Log.LogEvent(methodCode, eventPriority, System.Diagnostics.TraceEventType.Critical, _applicationName, methodName, string.Format("Message: {0}. InnerException: {1}.", ex.Message, ex.InnerException != null ? ex.InnerException.ToString() : string.Empty));
            }

            return _response;
        }

        //public CreateUserResponse CreateUser(CreateUserRequest request)
        //{
        //    int methodCode = (int)MethodCode.CreateUser;
        //    string methodName = Enum.GetName(typeof(MethodCode), MethodCode.CreateUser);
        //    CreateUserResponse _response = new CreateUserResponse();

        //    try
        //    {
        //        if (OperationContext.Current.ServiceSecurityContext.PrimaryIdentity.IsAuthenticated)
        //        {
        //            if (request != null && request.User != null)
        //            {
        //                _response.CreateStatus = _provider.CreateUser(request.User);
        //            }
        //        }
        //    }
        //    catch (Exception ex)
        //    {
        //        Log.LogEvent(methodCode, eventPriority, System.Diagnostics.TraceEventType.Critical, _applicationName, methodName, string.Format("Message: {0}. InnerException: {1}.", ex.Message, ex.InnerException != null ? ex.InnerException.ToString() : string.Empty));
        //    }

        //    return _response;
        //}

        public void DeleteApplications(DeleteApplicationsRequest request)
        {
            int methodCode = (int)MethodCode.DeleteApplications;
            string methodName = Enum.GetName(typeof(MethodCode), MethodCode.DeleteApplications);            

            try
            {
                if (OperationContext.Current.ServiceSecurityContext.PrimaryIdentity.IsAuthenticated)
                {
                    if (request != null && request.ApplicationNames != null && request.ApplicationNames.Count > 0)
                    {
                        _provider.DeleteApplications(request.ApplicationNames);
                    }
                }
            }
            catch (Exception ex)
            {
                Log.LogEvent(methodCode, eventPriority, System.Diagnostics.TraceEventType.Critical, _applicationName, methodName, string.Format("Message: {0}. InnerException: {1}.", ex.Message, ex.InnerException != null ? ex.InnerException.ToString() : string.Empty));
            }


        }

        public void DeleteRoles(DeleteRolesRequest request)
        {
            int methodCode = (int)MethodCode.DeleteRoles;
            string methodName = Enum.GetName(typeof(MethodCode), MethodCode.DeleteRoles);

            try
            {
                if (OperationContext.Current.ServiceSecurityContext.PrimaryIdentity.IsAuthenticated)
                {
                    if (request != null && request.ApplicationName != null && request.RoleNames != null && request.RoleNames.Count > 0)
                    {
                        _provider.DeleteRoles(request.ApplicationName, request.RoleNames);
                    }
                }
            }
            catch (Exception ex)
            {
                Log.LogEvent(methodCode, eventPriority, System.Diagnostics.TraceEventType.Critical, _applicationName, methodName, string.Format("Message: {0}. InnerException: {1}.", ex.Message, ex.InnerException != null ? ex.InnerException.ToString() : string.Empty));
            }
        }

        public void DeleteUsers(DeleteUsersRequest request)
        {
            int methodCode = (int)MethodCode.DeleteApplications;
            string methodName = Enum.GetName(typeof(MethodCode), MethodCode.DeleteUsers);

            try
            {
                if (OperationContext.Current.ServiceSecurityContext.PrimaryIdentity.IsAuthenticated)
                {
                    if (request != null && request.ApplicationName != null && request.UserNames != null && request.UserNames.Count > 0)
                    {
                        _provider.DeleteUsers(request.ApplicationName, request.UserNames);
                    }
                }
            }
            catch (Exception ex)
            {
                Log.LogEvent(methodCode, eventPriority, System.Diagnostics.TraceEventType.Critical, _applicationName, methodName, string.Format("Message: {0}. InnerException: {1}.", ex.Message, ex.InnerException != null ? ex.InnerException.ToString() : string.Empty));
            }
        }

        public GetApplicationsResponse GetApplications(GetApplicationsRequest request)
        {
            int methodCode = (int)MethodCode.GetApplications;
            string methodName = Enum.GetName(typeof(MethodCode), MethodCode.GetApplications);
            GetApplicationsResponse _response = new GetApplicationsResponse();

            try
            {
                if (OperationContext.Current.ServiceSecurityContext.PrimaryIdentity.IsAuthenticated)
                {
                    _response.Applications = _provider.GetApplications();
                }
            }
            catch (Exception ex)
            {
                Log.LogEvent(methodCode, eventPriority, System.Diagnostics.TraceEventType.Critical, _applicationName, methodName, string.Format("Message: {0}. InnerException: {1}.", ex.Message, ex.InnerException != null ? ex.InnerException.ToString() : string.Empty));
            }

            return _response;
        }

        public GetAuthRolesResponse GetAuthRoles(GetAuthRolesRequest request)
        {
            int methodCode = (int)MethodCode.GetAuthRoles;
            string methodName = Enum.GetName(typeof(MethodCode), MethodCode.GetAuthRoles);
            GetAuthRolesResponse _response = new GetAuthRolesResponse();

            try
            {
                if (OperationContext.Current.ServiceSecurityContext.PrimaryIdentity.IsAuthenticated)
                {
                    if(request != null)
                    {
                        _response.Roles = _provider.GetRoles(request.RoleNames, request.ApplicationName, request.ApplicationId, request.UserName);
                    }
                }
            }
            catch (Exception ex)
            {
                Log.LogEvent(methodCode, eventPriority, System.Diagnostics.TraceEventType.Critical, _applicationName, methodName, string.Format("Message: {0}. InnerException: {1}.", ex.Message, ex.InnerException != null ? ex.InnerException.ToString() : string.Empty));
            }

            return _response;
        }

        public GetAuthUsersResponse GetAuthUsers(GetAuthUsersRequest request)
        {
            int methodCode = (int)MethodCode.GetAuthUsers;
            string methodName = Enum.GetName(typeof(MethodCode), MethodCode.GetAuthUsers);
            GetAuthUsersResponse _response = new GetAuthUsersResponse();

            try
            {
                if (OperationContext.Current.ServiceSecurityContext.PrimaryIdentity.IsAuthenticated)
                {
                    if (request != null)
                    {
                        _response.Users = _provider.GetUsers(request.ApplicationName, request.RoleName);
                    }
                }
            }
            catch (Exception ex)
            {
                Log.LogEvent(methodCode, eventPriority, System.Diagnostics.TraceEventType.Critical, _applicationName, methodName, string.Format("Message: {0}. InnerException: {1}.", ex.Message, ex.InnerException != null ? ex.InnerException.ToString() : string.Empty));
            }

            return _response;
        }

       
        #endregion
    }
}
